# Generated by Django 2.1.4 on 2018-12-12 02:52

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0003_file__overwrite")]

    operations = [
        migrations.AddField(
            model_name="file",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                verbose_name="created at",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="file",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, verbose_name="updated at"),
        ),
        migrations.AddField(
            model_name="folder",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                verbose_name="created at",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="folder",
            name="updated_at",
            field=models.DateTimeField(auto_now=True, verbose_name="updated at"),
        ),
    ]



from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.db import models
from django.db.models import Q, signals
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _

from cabinet.base import (
    AbstractFile,
    DownloadMixin,
    ImageMixin,
    OverwriteMixin,
    TimestampsMixin,
)
from tree_queries.models import TreeNode


if not hasattr(settings, "CABINET_FILE_MODEL"):  # pragma: no branch
    settings.CABINET_FILE_MODEL = "cabinet.File"


def get_file_model():
    """
    Return the File model that is active in this project.
    """
    try:
        return apps.get_model(settings.CABINET_FILE_MODEL, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "CABINET_FILE_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "CABINET_FILE_MODEL refers to model '%s'"
            " that has not been installed" % settings.CABINET_FILE_MODEL
        )


class Folder(TimestampsMixin, TreeNode):
    name = models.CharField(_("name"), max_length=100)

    class Meta:
        ordering = ["name"]
        unique_together = [("parent", "name")]
        verbose_name = _("folder")
        verbose_name_plural = _("folders")

    def __str__(self):
        return self.name

    def clean(self):
        super().clean()
        if (
            not self.parent_id
            and Folder.objects.filter(~Q(pk=self.pk), Q(name=self.name)).exists()
        ):
            raise ValidationError(
                {"name": _("Root folder with same name exists already.")}
            )

    def ancestors_including_self(self):
        return self.ancestors(include_self=True)


class File(AbstractFile, ImageMixin, DownloadMixin, OverwriteMixin):
    FILE_FIELDS = ["image_file", "download_file"]

    caption = models.CharField(_("caption"), max_length=1000, blank=True)
    copyright = models.CharField(_("copyright"), max_length=1000, blank=True)

    class Meta(AbstractFile.Meta):
        swappable = "CABINET_FILE_MODEL"


@receiver(signals.post_delete, sender=File)
def delete_files(sender, instance, **kwargs):
    instance.delete_files()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-20 06:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="file",
            name="caption",
            field=models.CharField(blank=True, max_length=1000, verbose_name="caption"),
        ),
        migrations.AddField(
            model_name="file",
            name="copyright",
            field=models.CharField(
                blank=True, max_length=1000, verbose_name="copyright"
            ),
        ),
        migrations.AddField(
            model_name="file",
            name="image_alt_text",
            field=models.CharField(
                blank=True, max_length=1000, verbose_name="alternative text"
            ),
        ),
    ]



from urllib.parse import urlencode

import django
from django import forms
from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.contrib.admin.options import IncorrectLookupParameters
from django.contrib.admin.utils import get_deleted_objects
from django.contrib.admin.views.main import SEARCH_VAR
from django.core.exceptions import FieldDoesNotExist, PermissionDenied, ValidationError
from django.db import router, transaction
from django.db.models import Count
from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.text import capfirst
from django.utils.translation import gettext_lazy as _

from cabinet.models import Folder
from tree_queries.forms import TreeNodeChoiceField


class FolderListFilter(admin.RelatedFieldListFilter):
    """
    Filters are hidden in the file changelist; this filter is only responsible
    for filtering files by folders.
    """

    def has_output(self):
        return True

    def queryset(self, request, queryset):
        if "q" in request.GET:
            folder = self.used_parameters.get("folder__id__exact")
            if folder:
                return queryset.filter(
                    # Avoid problems because of table aliasses (Django 1.11)
                    folder__in=list(
                        Folder.objects.descendants(
                            folder, include_self=True
                        ).values_list("id", flat=True)
                    )
                )
            return queryset

        if self.used_parameters:
            try:
                return queryset.filter(**self.used_parameters)
            except ValidationError as e:
                raise IncorrectLookupParameters(e)
        else:
            return queryset.none()  # No files in root folder, never.


class FileTypeFilter(admin.SimpleListFilter):
    parameter_name = "file_type"
    title = _("file type")

    def lookups(self, request, model_admin):
        return [
            (row["file_field"], row["verbose_name"])
            for row in model_admin.model._file_mixin_fieldsets
        ]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.exclude(**{self.value(): ""})
        return queryset


class FolderForm(forms.ModelForm):
    class Meta:
        model = Folder
        fields = ("parent", "name")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields["_delete_folder"] = forms.BooleanField(
                required=False, label=_("Delete this folder")
            )


class SelectFolderForm(forms.Form):
    folder = TreeNodeChoiceField(
        queryset=Folder.objects.all(),
        label=capfirst(_("folder")),
        widget=forms.RadioSelect,
        empty_label=None,
    )

    def __init__(self, *args, **kwargs):
        files = kwargs.pop("files")
        super().__init__(*args, **kwargs)

        self.fields["files"] = forms.ModelMultipleChoiceField(
            queryset=files,
            label=capfirst(_("files")),
            initial=[f.id for f in files],
            widget=forms.CheckboxSelectMultiple,
        )
        self.order_fields(["files", "folder"])


def cabinet_querystring(request, **kwargs):
    values = {
        key: value
        for key, value in request.GET.items()
        if key not in {"folder__id__exact", "p"}
    }
    values.update(kwargs)
    return urlencode(sorted(values.items()))


class FolderAdminMixin(admin.ModelAdmin):
    def get_urls(self):
        from django.conf.urls import url

        return [
            url(
                r"^folder/add/$",
                self.admin_site.admin_view(self.folder_add),
                name="cabinet_folder_add",
            ),
            url(
                r"^folder/select/$",
                self.admin_site.admin_view(self.folder_select),
                name="cabinet_folder_select",
            ),
            url(
                r"^folder/(.+)/$",
                self.admin_site.admin_view(self.folder_change),
                name="cabinet_folder_change",
            ),
        ] + super().get_urls()

    def folder_add(self, request):
        with transaction.atomic(using=router.db_for_write(self.model)):
            return self._folder_form(
                request, {"initial": {"parent": request.GET.get("parent")}}
            )

    def folder_change(self, request, object_id):
        with transaction.atomic(using=router.db_for_write(self.model)):
            return self._folder_form(
                request, {"instance": get_object_or_404(Folder, pk=object_id)}
            )

    def _folder_form(self, request, kw):
        original = kw.get("instance")
        add = not original

        if add:
            if not self.has_add_permission(request):
                raise PermissionDenied
        else:
            if not self.has_change_permission(request, original):
                raise PermissionDenied

        if request.method == "POST":
            form = FolderForm(request.POST, **kw)
            if form.is_valid():
                if original and form.cleaned_data.get("_delete_folder"):
                    return self._folder_form_delete(request, original)

                folder = form.save()
                if original:
                    self.message_user(
                        request,
                        _('The folder "%s" was changed successfully.') % (folder,),
                        messages.SUCCESS,
                    )
                    return self.redirect_to_folder(request, folder.parent_id)

                else:
                    self.message_user(
                        request,
                        _('The folder "%s" was added successfully.') % (folder,),
                        messages.SUCCESS,
                    )
                    return self.redirect_to_folder(request, folder.id)

        else:
            form = FolderForm(**kw)

        adminForm = helpers.AdminForm(
            form,
            [[None, {"fields": list(form.fields.keys())}]],
            {},
            (),
            model_admin=self,
        )

        # render_change_form sets request.current_app
        response = self.render_change_form(
            request,
            dict(
                self.admin_site.each_context(request),
                title=(_("Add %s") if add else _("Change %s"))
                % Folder._meta.verbose_name,
                adminform=adminForm,
                inline_admin_formsets=[],
                object_id=original.pk if original else None,
                original=original,
                is_popup=False,
                media=self.media + adminForm.media,
                errors=helpers.AdminErrorList(form, []),
                preserve_filters=self.get_preserved_filters(request),
                cabinet={"querystring": cabinet_querystring(request)},
            ),
            add=add,
            change=not add,
            form_url=request.get_full_path(),
            obj=original,
        )
        response.template_name = [
            "admin/cabinet/folder/change_form.html",
            "admin/change_form.html",
        ]
        return response

    def _folder_form_delete(self, request, obj):
        if not self.has_delete_permission(request, obj):
            raise PermissionDenied

        using = router.db_for_write(obj.__class__)

        # Populate deleted_objects, a data structure of all related objects
        # that will also be deleted.
        if django.VERSION < (2, 1):
            (
                deleted_objects,
                model_count,
                perms_needed,
                protected,
            ) = get_deleted_objects(  # noqa
                [obj], obj._meta, request.user, self.admin_site, using
            )
        else:
            (
                deleted_objects,
                model_count,
                perms_needed,
                protected,
            ) = self.get_deleted_objects(
                [obj], request
            )  # noqa

        if protected or perms_needed:
            self.message_user(
                request,
                _("Cannot delete %(name)s") % {"name": obj._meta.verbose_name},
                messages.ERROR,
            )

        else:
            obj.delete()
            self.message_user(
                request,
                _('The folder "%s" was deleted successfully.') % obj,
                messages.SUCCESS,
            )

        return self.redirect_to_folder(request, obj.parent_id)

    def redirect_to_folder(self, request, folder_id):
        info = self.model._meta.app_label, self.model._meta.model_name
        url = reverse("admin:%s_%s_changelist" % info, current_app=self.admin_site.name)
        querydict = [
            (key, value)
            for key, value in request.GET.items()
            if key not in {"files", "folder__id__exact", "p", "parent"}
        ]
        if folder_id:
            querydict.append(("folder__id__exact", folder_id))
        return HttpResponseRedirect(
            "%s%s%s" % (url, "?" if querydict else "", urlencode(sorted(querydict)))
        )

    def move_to_folder(self, request, queryset):
        return HttpResponseRedirect(
            "%s?%s"
            % (
                reverse(
                    "admin:cabinet_folder_select", current_app=self.admin_site.name
                ),
                urlencode(sorted(("files", item.id) for item in queryset)),
            )
        )

    move_to_folder.short_description = _("Move files to folder")

    def folder_select(self, request):
        files = self.model.objects.filter(
            pk__in=(request.POST.getlist("files") or request.GET.getlist("files"))
        )

        form = SelectFolderForm(
            request.POST if request.method == "POST" else None, files=files
        )

        if form.is_valid():
            folder = form.cleaned_data["folder"]
            form.cleaned_data["files"].update(folder=folder)
            self.message_user(request, _("The files have been successfully moved."))
            return self.redirect_to_folder(request, folder.id)

        adminForm = helpers.AdminForm(
            form,
            [[None, {"fields": list(form.fields.keys())}]],
            {},
            (),
            model_admin=self,
        )

        response = self.render_change_form(
            request,
            dict(
                self.admin_site.each_context(request),
                title=_("Move files to folder"),
                adminform=adminForm,
                inline_admin_formsets=[],
                object_id=None,
                original=None,
                is_popup=False,
                media=self.media + adminForm.media,
                errors=helpers.AdminErrorList(form, []),
                preserve_filters=self.get_preserved_filters(request),
                cabinet={"querystring": cabinet_querystring(request)},
            ),
            add=False,
            change=False,
            form_url=request.get_full_path(),
            obj=None,
        )
        response.template_name = [
            "admin/cabinet/folder/change_form.html",
            "admin/change_form.html",
        ]
        return response


class IgnoreChangedDataErrorsForm(forms.ModelForm):
    """
    Ignore ``OSError`` exceptions when listing changed fields.

    Admin's construct_change_message runs after OverwriteMixin.save()
    and crashes when files are already gone. Simply ignore this;
    it makes admin log messages look less nice than they should, but
    we do not care too much.
    """

    @cached_property
    def changed_data(self):
        try:
            return super().changed_data
        except OSError:
            return []


class UploadForm(forms.Form):
    folder = forms.ModelChoiceField(queryset=Folder.objects.all())
    file = forms.FileField()


class FileAdminBase(FolderAdminMixin):
    actions = ["move_to_folder"]
    form = IgnoreChangedDataErrorsForm
    list_filter = [("folder", FolderListFilter), FileTypeFilter]
    search_fields = ("file_name",)

    # Useful when swapping the file model
    change_form_template = "admin/cabinet/file/change_form.html"
    change_list_template = "admin/cabinet/file/change_list.html"

    class Media:
        css = {"all": ("cabinet/cabinet.css",)}
        js = ["admin/js/jquery.init.js", "cabinet/cabinet.js"]

    def get_urls(self):
        from django.conf.urls import url

        return [
            url(
                r"^upload/$",
                self.admin_site.admin_view(self.upload),
                name="cabinet_upload",
            )
        ] + super().get_urls()

    def folders_annotate_counts(self, folders):
        """
        Add direct subfolders and files counts to an iterable of folders

        Recursive traversal and summation isn't implemented as adjacency
        list-based tree traversal without common table expressions is
        expensive. We want to stay compatible even with stupid database
        engines!
        """
        num_subfolders = dict(
            Folder.objects.order_by()
            .filter(parent__in=folders)
            .values("parent")
            .annotate(Count("id"))
            .values_list("parent", "id__count")
        )

        num_files = dict(
            self.model._default_manager.order_by()
            .filter(folder__in=folders)
            .values("folder")
            .annotate(Count("id"))
            .values_list("folder", "id__count")
        )

        for f in folders:
            f.num_subfolders = num_subfolders.get(f.id, 0)
            f.num_files = num_files.get(f.id, 0)

        return folders

    def changelist_view(self, request):
        folder__id__exact = request.GET.get("folder__id__exact")
        if folder__id__exact == "last":
            kw = {}
            if request.COOKIES.get("cabinet_folder"):
                kw["folder__id__exact"] = request.COOKIES["cabinet_folder"]
            return HttpResponseRedirect(
                "?{}".format(cabinet_querystring(request, **kw))
            )

        cabinet_context = {
            # Keep query params except those in the set below when changing
            # folders
            "querystring": cabinet_querystring(request)
        }

        folder = None

        # Never filter by folder if searching
        if not request.GET.get(SEARCH_VAR):
            if folder__id__exact:
                try:
                    folder = Folder.objects.get(pk=folder__id__exact)
                except (Folder.DoesNotExist, ValueError):
                    return HttpResponseRedirect("?e=1")

            if folder is None:
                cabinet_context.update(
                    {
                        "folder": None,
                        "folder_children": self.folders_annotate_counts(
                            Folder.objects.filter(parent__isnull=True)
                        ),
                    }
                )

            else:
                cabinet_context.update(
                    {
                        "folder": folder,
                        "folder_children": self.folders_annotate_counts(
                            folder.children.all()
                        ),
                    }
                )

        response = super().changelist_view(
            request,
            extra_context={
                "cabinet": cabinet_context,
                "title": folder or _("Root folder"),
            },
        )
        response.set_cookie("cabinet_folder", folder.pk if folder else "")
        return response

    def add_view(self, request, form_url="", extra_context=None):
        extra_context = extra_context or {}
        if request.GET.get("folder"):
            extra_context["cabinet"] = {
                "folder": Folder.objects.filter(pk=request.GET["folder"]).first()
            }

        return self.changeform_view(
            request, None, request.get_full_path(), extra_context
        )

    def _add_folder_if_redirect(self, response, folder_id):
        # Keep the folder preset when redirecting. This sometimes adds the
        # folder variable twice (once to preserved_filters and once separately
        # but this is at most ugly and not a real problem)
        if response.status_code == 302 and folder_id:
            response["Location"] += "{}folder__id__exact={}".format(
                "&" if "?" in response["Location"] else "?", folder_id
            )
        return response

    def response_add(self, request, obj, **kwargs):
        return self._add_folder_if_redirect(
            super().response_add(request, obj, **kwargs), obj.folder_id
        )

    def response_change(self, request, obj, **kwargs):
        return self._add_folder_if_redirect(
            super().response_change(request, obj, **kwargs), obj.folder_id
        )

    def upload(self, request):
        if request.method != "POST":
            return self.redirect_to_folder(request, None)
        form = UploadForm(request.POST, request.FILES)
        if not form.is_valid():
            return JsonResponse({"success": False}, status=400)

        f = self.model(folder=form.cleaned_data["folder"])
        f.file = form.cleaned_data["file"]
        f.save()

        return JsonResponse({"success": True, "pk": f.pk, "name": str(f)})

    top_fields = ["folder", "caption", "copyright"]
    advanced_fields = ["_overwrite"]

    def get_fieldsets(self, request, obj=None):
        def exists(field):
            try:
                self.model._meta.get_field(field)
            except FieldDoesNotExist:
                return False
            return True

        fieldsets = [
            (None, {"fields": [field for field in self.top_fields if exists(field)]})
        ]
        for row in self.model._file_mixin_fieldsets:
            if getattr(obj, row["file_field"], None):
                fieldsets[0][1]["fields"].extend(row["fields"])
            else:
                fieldsets.append(
                    (
                        row["verbose_name"],
                        {
                            "fields": row["fields"],
                            "classes": ["collapse"] if obj else [],
                        },
                    )
                )

        advanced = [field for field in self.advanced_fields if exists(field)]
        if advanced:
            fieldsets.append(
                (_("Advanced"), {"fields": advanced, "classes": ["collapse"]})
            )
        return fieldsets



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-18 12:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import imagefield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="File",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "image_file",
                    imagefield.fields.ImageField(
                        blank=True,
                        height_field="image_height",
                        upload_to="cabinet/%Y/%m",
                        verbose_name="image",
                        width_field="image_width",
                    ),
                ),
                (
                    "image_width",
                    models.PositiveIntegerField(
                        blank=True,
                        editable=False,
                        null=True,
                        verbose_name="image width",
                    ),
                ),
                (
                    "image_height",
                    models.PositiveIntegerField(
                        blank=True,
                        editable=False,
                        null=True,
                        verbose_name="image height",
                    ),
                ),
                (
                    "image_ppoi",
                    imagefield.fields.PPOIField(
                        default="0.5x0.5",
                        max_length=20,
                        verbose_name="primary point of interest",
                    ),
                ),
                (
                    "download_file",
                    models.FileField(
                        blank=True, upload_to="cabinet/%Y/%m", verbose_name="download"
                    ),
                ),
                (
                    "download_type",
                    models.CharField(
                        editable=False, max_length=20, verbose_name="download type"
                    ),
                ),
                (
                    "file_name",
                    models.CharField(max_length=1000, verbose_name="file name"),
                ),
                ("file_size", models.PositiveIntegerField(verbose_name="file size")),
            ],
            options={
                "abstract": False,
                "verbose_name": "file",
                "verbose_name_plural": "files",
                "ordering": ["file_name"],
                "swappable": "CABINET_FILE_MODEL",
            },
        ),
        migrations.CreateModel(
            name="Folder",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
                (
                    "parent",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="cabinet.Folder",
                        verbose_name="parent",
                    ),
                ),
            ],
            options={
                "ordering": ["name"],
                "verbose_name": "folder",
                "verbose_name_plural": "folders",
            },
        ),
        migrations.AddField(
            model_name="file",
            name="folder",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="files",
                to="cabinet.Folder",
                verbose_name="folder",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="folder", unique_together=set([("parent", "name")])
        ),
    ]



from django import forms
from django.conf import settings
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import NoReverseMatch, reverse
from django.utils.text import Truncator
from django.utils.translation import gettext_lazy as _

from cabinet.models import Folder
from tree_queries.forms import TreeNodeChoiceField


class UploadForm(forms.Form):
    folder = TreeNodeChoiceField(label=_("folder"), queryset=Folder.objects.all())
    file = forms.FileField(label=_("file"))


class CabinetFileRawIdWidget(ForeignKeyRawIdWidget):
    template_name = "admin/cabinet/cabinet_file_raw_id_widget.html"

    class Media:
        css = {"all": ["cabinet/inline-upload.css"]}
        js = ["admin/js/jquery.init.js", "cabinet/inline-upload.js"]

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        instance = getattr(self, "instance", None)
        context["cabinet"] = {
            "upload_form": UploadForm(
                prefix="cu-{}".format(id(self)),
                initial={"folder": instance and instance.folder_id},
            ),
            "instance": instance,
        }
        context["related_url"] += "&folder__id__exact={}".format(
            instance.folder_id if instance else "last"
        )
        return context

    def label_and_url_for_value(self, value):
        # Copied from django/contrib/admin/widgets.py with the addition of
        # saving the obj as self.instance
        key = self.rel.get_related_field().name
        try:
            obj = self.rel.model._default_manager.using(self.db).get(**{key: value})
        except (ValueError, self.rel.model.DoesNotExist, ValidationError):
            obj = None
            return "", ""

        try:
            url = reverse(
                "admin:%s_%s_change"
                % (obj._meta.app_label, obj._meta.object_name.lower()),
                args=(obj.pk,),
                current_app=self.admin_site.name,
            )
        except NoReverseMatch:
            url = ""  # Admin not registered for target model.

        self.instance = obj
        return Truncator(obj).words(14, truncate="..."), url


class CabinetForeignKey(models.ForeignKey):
    def __init__(self, to=None, **kwargs):
        super().__init__(to or settings.CABINET_FILE_MODEL, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        return (name, "django.db.models.ForeignKey", args, kwargs)

    def formfield(self, **kwargs):
        widget = kwargs.get("widget")
        if widget and isinstance(widget, ForeignKeyRawIdWidget):
            widget.__class__ = CabinetFileRawIdWidget
        return super().formfield(**kwargs)



from django import forms
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
from django.utils.html import format_html


class CKEditorFilebrowserMixin(admin.ModelAdmin):
    def get_changelist(self, request, **kwargs):
        if request.GET.get("CKEditorFuncNum"):
            return CKFileBrowserChangeList
        return ChangeList

    @property
    def media(self):
        return super().media + forms.Media(js=["cabinet/ckeditor.js"])


class CKFileBrowserChangeList(ChangeList):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.list_display = [
            Link(self, field) if field in self.list_display_links else field
            for field in self.list_display
        ]
        self.list_display_links = None

    def get_filters_params(self, params=None):
        params = super().get_filters_params(params)
        self.ck_context = {
            "CKEditor": params.pop("CKEditor"),
            "CKEditorFuncNum": params.pop("CKEditorFuncNum"),
            "langCode": params.pop("langCode"),
        }
        return params


class Link(object):
    def __init__(self, cl, name):
        self.cl = cl
        self.name = name
        self.__name__ = name

    def __call__(self, obj):
        # Currently, only supports callables (sufficient for admin_thumbnail
        # and admin_file_name)
        fn = getattr(self.cl.model_admin, self.name)
        result = fn(obj)
        return format_html(
            '<a href="{url}" data-ckeditor-function="{num}">{result}</a>',
            url=obj.file.url,
            num=self.cl.ck_context["CKEditorFuncNum"],
            result=result,
        )

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-20 02:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0002_auto_20170620_0846")]

    operations = [
        migrations.AddField(
            model_name="file",
            name="_overwrite",
            field=models.BooleanField(
                default=False,
                help_text="By default, a new and unique filename is generated for each file, which also helps with caching.",
                verbose_name="Keep filename this time when uploading new file?",
            ),
        )
    ]



from django.contrib import admin
from django.template.defaultfilters import filesizeformat
from django.utils.formats import date_format
from django.utils.html import format_html, format_html_join, mark_safe
from django.utils.translation import gettext_lazy as _

from cabinet.base_admin import FileAdminBase
from cabinet.ckeditor import CKEditorFilebrowserMixin
from cabinet.models import File


@admin.register(File)
class FileAdmin(CKEditorFilebrowserMixin, FileAdminBase):
    list_display = ["admin_thumbnail", "admin_file_name", "admin_details"]
    list_display_links = ["admin_thumbnail", "admin_file_name"]

    def admin_thumbnail(self, instance):
        if instance.image_file.name:
            try:
                target = instance.image_file.process(["default", ("crop", (50, 50))])
                return format_html(
                    '<img src="{}" alt=""/>', instance.image_file.storage.url(target)
                )
            except Exception:
                return mark_safe('<span class="broken-image"></span>')
        elif instance.download_file.name:
            return format_html(
                '<span class="download download-{}">{}</span>',
                instance.download_type,
                instance.download_type.upper(),
            )
        return ""

    admin_thumbnail.short_description = ""

    def admin_file_name(self, instance):
        return format_html(
            "{} <small>({})</small>",
            instance.file_name,
            filesizeformat(instance.file_size),
        )

    admin_file_name.short_description = _("file name")

    def admin_details(self, instance):
        details = [
            instance.caption,
            instance.copyright,
            _("Created %(created_at)s, last modified %(updated_at)s")
            % {
                "created_at": date_format(instance.created_at, "SHORT_DATE_FORMAT"),
                "updated_at": date_format(instance.updated_at, "SHORT_DATE_FORMAT"),
            },
        ]
        return format_html(
            "<small>{}</small>",
            format_html_join(mark_safe("<br>"), "{}", ((d,) for d in details if d)),
        )

    admin_details.short_description = _("details")



import inspect
import io
import os
import re
from PIL import Image

from django.core.exceptions import (
    FieldDoesNotExist,
    ImproperlyConfigured,
    ValidationError,
)
from django.db import models
from django.db.models import signals
from django.utils.translation import gettext_lazy as _

from imagefield.fields import ImageField, PPOIField
from tree_queries.fields import TreeNodeForeignKey


UPLOAD_TO = "cabinet/%Y/%m"


def upload_is_image(data):
    """
    Determine whether ``data`` is an image or not

    Usage::

        if upload_is_image(request.FILES['file']):
            ...
    """
    # From django/forms/fields.py
    if hasattr(data, "temporary_file_path"):
        file = data.temporary_file_path()
    else:
        file = io.BytesIO(data.read())

    try:
        image = Image.open(file)
        image.resize([20, 20], Image.LANCZOS)  # Verify image some more
        return True
    except Exception:  # OSError, errors while resizing
        return False


class ImageMixin(models.Model):
    image_file = ImageField(
        _("image"),
        upload_to=UPLOAD_TO,
        width_field="image_width",
        height_field="image_height",
        ppoi_field="image_ppoi",
        blank=True,
        max_length=1000,
    )
    image_width = models.PositiveIntegerField(
        _("image width"), blank=True, null=True, editable=False
    )
    image_height = models.PositiveIntegerField(
        _("image height"), blank=True, null=True, editable=False
    )
    image_ppoi = PPOIField(_("primary point of interest"))
    image_alt_text = models.CharField(
        _("alternative text"), max_length=1000, blank=True
    )

    class Meta:
        abstract = True
        verbose_name = _("image")
        verbose_name_plural = _("images")

    def accept_file(self, value):
        if upload_is_image(value):
            self.image_file = value
            return True


class DownloadMixin(models.Model):
    DOWNLOAD_TYPES = [  # pragma: no branch (last condition always matches)
        # Should we be using imghdr.what instead of extension guessing?
        (
            "image",
            _("Image"),
            lambda f: re.compile(
                r"\.(bmp|jpe?g|jp2|jxr|gif|png|tiff?)$", re.IGNORECASE
            ).search(f),
        ),
        (
            "video",
            _("Video"),
            lambda f: re.compile(
                r"\.(mov|m[14]v|mp4|avi|mpe?g|qt|ogv|wmv|flv)$", re.IGNORECASE
            ).search(f),
        ),
        (
            "audio",
            _("Audio"),
            lambda f: re.compile(
                r"\.(au|mp3|m4a|wma|oga|ram|wav)$", re.IGNORECASE
            ).search(f),
        ),
        ("pdf", _("PDF document"), lambda f: f.lower().endswith(".pdf")),
        ("swf", _("Flash"), lambda f: f.lower().endswith(".swf")),
        ("txt", _("Text"), lambda f: f.lower().endswith(".txt")),
        ("rtf", _("Rich Text"), lambda f: f.lower().endswith(".rtf")),
        ("zip", _("Zip archive"), lambda f: f.lower().endswith(".zip")),
        (
            "doc",
            _("Microsoft Word"),
            lambda f: re.compile(r"\.docx?$", re.IGNORECASE).search(f),
        ),
        (
            "xls",
            _("Microsoft Excel"),
            lambda f: re.compile(r"\.xlsx?$", re.IGNORECASE).search(f),
        ),
        (
            "ppt",
            _("Microsoft PowerPoint"),
            lambda f: re.compile(r"\.pptx?$", re.IGNORECASE).search(f),
        ),
        ("other", _("Binary"), lambda f: True),  # Must be last
    ]

    download_file = models.FileField(
        _("download"), upload_to=UPLOAD_TO, blank=True, max_length=1000
    )
    download_type = models.CharField(_("download type"), max_length=20, editable=False)

    class Meta:
        abstract = True
        verbose_name = _("download")
        verbose_name_plural = _("downloads")

    def save(self, *args, **kwargs):
        self.download_type = (  # pragma: no branch
            next(
                type
                for type, title, check in self.DOWNLOAD_TYPES
                if check(self.download_file.name)
            )
            if self.download_file
            else ""
        )
        super().save(*args, **kwargs)

    save.alters_data = True

    def accept_file(self, value):
        self.download_file = value
        return True


class OverwriteMixin(models.Model):
    _overwrite = models.BooleanField(
        _("Keep filename this time when uploading new file?"),
        default=False,
        help_text=_(
            "By default, a new and unique filename is generated for each file,"
            " which also helps with caching."
        ),
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        original = None
        if self.pk:
            original = self.__class__._base_manager.filter(pk=self.pk).first()

        if self._overwrite and original and not self.file._committed:
            original_file = original.file
            original_file_name = original_file.name
            original.delete_files()
            original_file.delete(save=False)

            new_file = self.file
            new_file.storage.save(
                original_file_name, new_file.file, max_length=new_file.field.max_length
            )
            new_file._committed = True
            setattr(self, new_file.field.name, original_file_name)

            # Better be safe than sorry:
            new_file.name = original_file_name
            self.file_name = os.path.basename(new_file.name)

            self._overwrite = False  # Only overwrite once.
            super().save(*args, **kwargs)

        else:
            self._overwrite = False  # Only overwrite once.
            super().save(*args, **kwargs)

            if original and (
                original.file.name != self.file.name
                or original.file.storage != self.file.storage
            ):
                original.delete_files()

    save.alters_data = True


class TimestampsMixin(models.Model):
    created_at = models.DateTimeField(_("created at"), auto_now_add=True)
    updated_at = models.DateTimeField(_("updated at"), auto_now=True)

    class Meta:
        abstract = True


class AbstractFile(TimestampsMixin):
    FILE_FIELDS = []

    folder = TreeNodeForeignKey(
        "cabinet.Folder",
        on_delete=models.PROTECT,
        verbose_name=_("folder"),
        related_name="files",
    )

    file_name = models.CharField(_("file name"), max_length=1000)
    file_size = models.PositiveIntegerField(_("file size"))

    class Meta:
        abstract = True
        ordering = ["file_name"]
        verbose_name = _("file")
        verbose_name_plural = _("files")

    def __str__(self):
        return self.file_name

    def __files(self):
        for field in self.FILE_FIELDS:
            f = getattr(self, field)
            if f and f.name:
                yield f

    @property
    def file(self):
        return next(self.__files())

    @file.setter
    def file(self, value):
        for fn in self._accept_file_functions:
            if fn(self, value):
                break
        else:  # pragma: no cover (improper configuration!)
            raise TypeError("Invalid value %r" % value)

    def clean(self):
        if len(list(self.__files())) != 1:
            raise ValidationError(_("Please fill in exactly one file field!"))

    def save(self, *args, **kwargs):
        f_obj = self.file
        self.file_name = os.path.basename(f_obj.name)
        self.file_size = f_obj.size
        super().save(*args, **kwargs)

    save.alters_data = True

    def delete_files(self):
        for field in self.FILE_FIELDS:
            f_obj = getattr(self, field)
            if not f_obj.name:
                continue

            # f_obj.storage.delete(f_obj.name)
            f_obj.delete(save=False)

    delete_files.alters_data = True


def determine_accept_file_functions(sender, **kwargs):
    if issubclass(sender, AbstractFile) and not sender._meta.abstract:
        fields = set(sender.FILE_FIELDS)
        fns = {}
        fieldsets = []

        for cls in list(inspect.getmro(sender))[1:]:
            for f in fields:
                try:
                    cls._meta.get_field(f)
                except FieldDoesNotExist:
                    pass
                else:
                    # File field exists on this class. There *must* be an
                    # accept_file method as well.
                    fns[f] = cls.accept_file
                    fields.discard(f)
                    fieldsets.append(
                        {
                            "verbose_name": cls._meta.verbose_name,
                            "fields": [
                                field.name
                                for field in cls._meta.local_fields
                                if field.editable
                            ],
                            "file_field": f,
                        }
                    )
                    break

        if fields:  # pragma: no cover
            raise ImproperlyConfigured(
                'No "accept_file" method found for %s' % (", ".join(sorted(fields)),)
            )

        sender._accept_file_functions = [fns[f] for f in sender.FILE_FIELDS]
        sender._file_mixin_fieldsets = fieldsets


signals.class_prepared.connect(determine_accept_file_functions)



# Generated by Django 2.2 on 2019-04-15 03:43

from django.db import migrations, models
import imagefield.fields


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0004_auto_20181212_0252")]

    operations = [
        migrations.AlterField(
            model_name="file",
            name="download_file",
            field=models.FileField(
                blank=True,
                max_length=1000,
                upload_to="cabinet/%Y/%m",
                verbose_name="download",
            ),
        ),
        migrations.AlterField(
            model_name="file",
            name="image_file",
            field=imagefield.fields.ImageField(
                blank=True,
                height_field="image_height",
                max_length=1000,
                upload_to="cabinet/%Y/%m",
                verbose_name="image",
                width_field="image_width",
            ),
        ),
    ]
